@@ -6,8 +6,11 @@ from tempfile import mkstemp
 import shutil
 import warnings
 
+from hypothesis.strategies import composite, integers, one_of
+from hypothesis import given
+
 from tests import TestCase, DATA_DIR, get_temp_copy
-from mutagen._compat import cBytesIO, text_type, xrange
+from mutagen._compat import cBytesIO, text_type
 from mutagen import File, Metadata, FileType, MutagenError, PaddingInfo
 from mutagen._util import loadfile, get_size
 from mutagen.oggvorbis import OggVorbis
@@ -29,6 +32,7 @@ from mutagen.asf import ASF
 from mutagen.aiff import AIFF
 from mutagen.aac import AAC
 from mutagen.smf import SMF
+from mutagen.dsf import DSF
 from os import devnull
 
 
@@ -260,7 +264,7 @@ class _TestFileObj(object):
         return self._fileobj.seek(offset, whence)
 
 
-def iter_test_file_objects(fileobj):
+def generate_test_file_objects(fileobj, func):
     """Given a file object yields the same file object which fails differently
     each time
     """
@@ -268,11 +272,23 @@ def iter_test_file_objects(fileobj):
     t = _TestFileObj(fileobj)
     # first figure out how much a successful attempt reads and how many
     # file object operations it executes.
-    yield t
-    for i in xrange(t.dataread):
-        yield _TestFileObj(fileobj, stop_after=i)
-    for i in xrange(t.operations):
-        yield _TestFileObj(fileobj, fail_after=i)
+    func(t)
+
+    @composite
+    def strategy(draw):
+
+        stop_strat = integers(
+            min_value=0, max_value=t.dataread).map(
+                lambda i: _TestFileObj(fileobj, stop_after=i))
+
+        fail_strat = integers(
+            min_value=0, max_value=t.operations).map(
+                lambda i: _TestFileObj(fileobj, fail_after=i))
+
+        x = draw(one_of(stop_strat, fail_strat))
+        return x
+
+    return strategy()
 
 
 class TAbstractFileType(object):
@@ -312,6 +328,8 @@ class TAbstractFileType(object):
         with open(self.filename, "rb") as h:
             fileobj = cBytesIO(h.read())
             self.KIND(fileobj)
+            # make sure it's not closed
+            fileobj.read(0)
 
     def test_testfileobj(self):
         with open(self.filename, "rb") as h:
@@ -319,30 +337,43 @@ class TAbstractFileType(object):
 
     def test_test_fileobj_load(self):
         with open(self.filename, "rb") as h:
-            for t in iter_test_file_objects(h):
+
+            @given(generate_test_file_objects(h, self.KIND))
+            def run(t):
                 try:
                     self.KIND(t)
                 except MutagenError:
                     pass
 
+            run()
+
     def test_test_fileobj_save(self):
         with open(self.filename, "rb+") as h:
             o = self.KIND(_TestFileObj(h))
-            for t in iter_test_file_objects(h):
+
+            @given(generate_test_file_objects(h, lambda t: o.save(fileobj=t)))
+            def run(t):
                 try:
                     o.save(fileobj=t)
                 except MutagenError:
                     pass
 
+            run()
+
     def test_test_fileobj_delete(self):
         with open(self.filename, "rb+") as h:
             o = self.KIND(_TestFileObj(h))
-            for t in iter_test_file_objects(h):
+
+            @given(generate_test_file_objects(
+                h, lambda t: o.delete(fileobj=t)))
+            def run(t):
                 try:
                     o.delete(fileobj=t)
                 except MutagenError:
                     pass
 
+            run()
+
     def test_filename(self):
         self.assertEqual(self.audio.filename, self.filename)
 
@@ -488,6 +519,12 @@ _FILETYPES = {
     SMF: [
         os.path.join(DATA_DIR, "sample.mid"),
     ],
+    DSF: [
+        os.path.join(DATA_DIR, '2822400-1ch-0s-silence.dsf'),
+        os.path.join(DATA_DIR, '5644800-2ch-s01-silence.dsf'),
+        os.path.join(DATA_DIR, 'with-id3.dsf'),
+        os.path.join(DATA_DIR, 'without-id3.dsf'),
+    ]
 }
 
 _FILETYPES[ID3FileType] = _FILETYPES[MP3]
@@ -548,12 +585,16 @@ class TFile(TestCase):
     def test_mock_fileobj(self):
         for filename in self.filenames:
             with open(filename, "rb") as h:
-                for t in iter_test_file_objects(h):
+
+                @given(generate_test_file_objects(h, File))
+                def run(t):
                     try:
                         File(t)
                     except MutagenError:
                         pass
 
+                run()
+
     def test_easy_mp3(self):
         self.failUnless(isinstance(
             File(os.path.join(DATA_DIR, "silence-44-s.mp3"), easy=True),
